#! /usr/bin/env bash

# logfile=$$.log
# exec > $logfile 2>&1

horizon_def=24
history_file_def=example_files/profiles.csv
past_length_def=10
past_distribution_def=`echo $past_length_def | awk '{sum = 0; pow_2 = 1; for (i = 1; i < $1; i++) {pow_2 /= 2; printf("%.30lf:", pow_2); sum += pow_2} printf("%.30lf", 1 - sum);}'`
time_stamp=`date +%F-%H-%M-%N`
#max_lsof=`lsof -i -P +c 100 2>&1 | head -1 | awk '{if ($1 == "COMMAND") print 100; else {gsub("\\\\(", ""); gsub("\\\\)", ""); print $NF}}'`

function usage ()
{
    echo $0 | awk -v horizon_def=${horizon_def} -v history_file_def=${history_file_def} -v past_length_def=${past_length_def} -v past_distribution_def=${past_distribution_def} '{printf("\n\n\nUsage: %s [-l horizon] [-f history_file] [-p past_length_in_days] [-pd past_distribution]\n\n\nWhere:\n\thorizon is how many hours have to be forecasted (default is %s)\n\thistory_file is a CSV file with the power demand history (default is %s, which may be used as an example for the format)\n\tpast_length is the number of days in the past (default is %s)\n\tpast_distribution is the discounting factor for the days in the past. Format is x1:...:xn, being n=past_length, and the summation on all xi must be 1 (default is %s)\n\n\n", $1, horizon_def, history_file_def, past_length_def, past_distribution_def);}'
    exit
}

function KillHouseForSure()
{
    # only_first=`echo $max_lsof House.$EXP_ID | awk '{print substr($2, 1, $1);}'`
    pidThis=`ps ax | grep -F House.$EXP_ID | grep -v grep | awk '{print $1;}'`
    if [[ "$pidThis" != "" ]]
    then
	kill -9 $pidThis >/dev/null 2>&1
    fi
    sleep 60
}

function cp_fail()
{
    if [ ! \( -f $1 \) ]
    then
	echo Unable to find required file $1, exiting
	exit
    fi
    cp $1 $2
}

function Experiment() 
{

    past_length=$1
    horizon=$2
    history_file=$3
    past_distribution=$4
    EXP_ID=$time_stamp

    echo -n "Initializing..."
    if [ ! \( -d aux_files \) ]
    then
	echo Unable to find directory aux_files
	exit
    fi
    mkdir -p results
    mkdir -p results/$EXP_ID
    mkdir -p results/$EXP_ID/input
    mkdir -p results/$EXP_ID/bin
    mkdir -p results/$EXP_ID/output
    if [ ! \( -x src/EUMF-K/bin/EUMF-K.AHEAD$horizon \) -o ! \( -x src/House/bin/House \) ]
    then
	make -C src HORIZON=$horizon > src/make.log 2>&1
	if [ ! \( -x src/EUMF-K/bin/EUMF-K.AHEAD$horizon \) -o ! \( -x src/House/bin/House \) ]
	then
	    echo Unable to compile, exiting
	    exit
	fi
    fi
    cp src/House/bin/House results/$EXP_ID/bin/House.$EXP_ID
    cp src/EUMF-K/bin/EUMF-K.AHEAD$horizon results/$EXP_ID/bin/EUMF-K
    cat $history_file | awk '{if (FNR > 1) print $0",,,0"; else print;}' > results/$EXP_ID/input/`basename $history_file`
    cp_fail aux_files/pricepolicies.csv results/$EXP_ID/input
    cp_fail aux_files/battery0.csv results/$EXP_ID/input
    cp_fail aux_files/phev.csv results/$EXP_ID/input
    cp_fail aux_files/energy.csv results/$EXP_ID/input
    cp_fail aux_files/House_conf.ini results/$EXP_ID/input
    cp_fail aux_files/HECH_conf.ini results/$EXP_ID/input
    ports=`grep PORT aux_files/HECH_conf.ini | awk '{print $NF}'`
    # only_first=`echo $max_lsof House.$EXP_ID | awk '{print substr($2, 1, $1);}'`
    try="Error"
    while [ "$try" != "" ]
    do

	# # while [ "`lsof -i -P +c 0 | grep -i listen | grep $only_first`" != "" ]
	# while [ "`lsof -i -P +c 0 | grep -i listen | grep -F House.`" != "" ]
	# do
	#     KillHouseForSure
	# done
	for port in $ports
	do
	    res=`lsof -i -P +c 0 | grep -i listen | grep -F ":$port "`
	    if [ "${res}" != "" ]
	    then
		kill -9 `echo $res | awk '{print $2}'`
	    fi
	    sleep 60
	done
	test -f House_conf.ini || ln -s results/$EXP_ID/input/House_conf.ini
	test -f HECH_conf.ini || ln -s results/$EXP_ID/input/HECH_conf.ini
 	./results/$EXP_ID/bin/House.$EXP_ID --profilesFile results/$EXP_ID/input/`basename $history_file` --loggingFile results/$EXP_ID/output/house_output.csv --batteryFile results/$EXP_ID/input/battery0.csv --phevFile results/$EXP_ID/input/phev.csv > results/$EXP_ID/output/house.log 2>&1 &

	sleep 60

	try=`grep "Address already in use" results/$EXP_ID/output/house.log`
    done

    pastDistribution=$past_distribution
    pastLength=$past_length
    EUMF="./results/$EXP_ID/bin/EUMF-K"
    tool="EBR-CTRL"
    tariff="IBR"

    echo " done"
    echo -n "Running... "
    $EUMF --tool $tool --tariff $tariff --energyFile results/$EXP_ID/input/energy.csv --loggingFile results/$EXP_ID/output/results.all.csv --batteryFile results/$EXP_ID/input/battery0.csv --phevFile results/$EXP_ID/input/phev.csv --pastLength $pastLength --pastDistribution $pastDistribution  --pricePolicyFile results/$EXP_ID/input/pricepolicies.csv --useHistoryFile results/$EXP_ID/input/`basename $history_file` > results/$EXP_ID/output/eumf_k.log 2>&1
    awk -v horizon=$horizon -F, '{if (ARGIND == 1) {if (FNR > 1) timestamps[FNR] = sprintf("%s,%s", $1, $2);} else if (FNR == 1) {printf("Start,End,Consumption (kW),Production (kW),Demand (kW),"); for (i = 1; i < horizon; i++) printf("Forecasted consumption after %d hours (kW),Forecasted production after %d hours (kW),Forecasted demand after %d hours (kW),", i, i, i);} else {printf("\n%s,%s,%s,%s,", timestamps[FNR], $3, $4, $1); for (i = 1; i < horizon; i++) printf("%s,%s,%lf,", $(14 + (i - 1)*3), $(14 + 1 + (i - 1)*3), $(14 + (i - 1)*3) - $(14 + 1 + (i - 1)*3));}}' results/$EXP_ID/input/`basename $history_file` results/$EXP_ID/output/results.all.csv > results/$EXP_ID/output/results.csv
    rm -f House_conf.ini
    rm -f HECH_conf.ini
    rm -f results/$EXP_ID/output/results.all.csv results/$EXP_ID/output/house_output.csv
    echo " done"
    echo "See results/$EXP_ID/output/results.csv for the experiment result"
}

function FileDoesNotExist ()
{
    if [ ! \( -f $1 \) ]
    then
	echo Error: file $1 does not exist
	exit
    fi
}

min_num_args=0
max_num_args=8

if [ $# -lt ${min_num_args} -o $# -gt ${max_num_args} ]
then
	usage
fi

history_file=${history_file_def}
horizon=${horizon_def}
past_length=${past_length_def}
past_distribution=${past_distribution_def}
args_ptr=1
while [ $# -ge ${args_ptr} ]
do
    opt=`echo "$*" | awk -v arg=${args_ptr} '{print $arg}'`
    num_opts=2
    case "${opt}" in
	"-l") horizon=`echo "$*" | awk -v arg=${args_ptr} '{print $(arg + 1) + 1}'`;;
	"-f") history_file=`echo "$*" | awk -v arg=${args_ptr} '{print $(arg + 1)}'`;;
	"-p") past_length=`echo "$*" | awk -v arg=${args_ptr} '{print $(arg + 1)}'`;;
	"-pd") past_distribution=`echo "$*" | awk -v arg=${args_ptr} '{print $(arg + 1)}'`;;
	"-h") usage;;
	*) 
	    echo Unrecognized option `echo "$*" | awk -v arg=${args_ptr} '{print $arg}'`
	    usage;;
    esac
    ((args_ptr+=num_opts))
done

FileDoesNotExist ${history_file}
if [ $horizon -gt 25 ]
then
    echo Parameter -l must be at most 24
    exit
fi

Experiment $past_length $horizon $history_file $past_distribution
