#include <EUMF-K.h>
#include <Debug.h>
#include <StringUtils.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>


#define DEBUG "EBR"




double EBR_energyPrice(EBR *_this, double energy, double * energyPrice)
{
	Debug_assert(DEBUG, _this != NULL, "_this == NULL\n");
	double res = 0;
	int hour = _this->currentHour;

	if (_this->tariff == TARIFF_IBR) {
		if (energy >= 0) {
			if (energy > _this->pricePolicy[PRICEPOLICY_MAX][0]) {
				* energyPrice = energy * _this->IBR[IBR_BUY_HIGH];
				res = _this->IBR[IBR_BUY_HIGH];
			}
			else {
				* energyPrice = energy * _this->IBR[IBR_BUY_LOW];
				res = _this->IBR[IBR_BUY_LOW];
			}
		}
		else {
			* energyPrice = energy * _this->IBR[IBR_SELL];
			res = _this->IBR[IBR_SELL];
		}
		Debug_out(DEBUG, "EBR_energyPrice (IBR): energy = %lg, max = %lg, tariff = %lg, price = %lg\n", energy, _this->pricePolicy[PRICEPOLICY_MAX][0], res, *energyPrice);
	}
	else if (_this->tariff == TARIFF_TOU) {
		if (energy >= 0) {
			*energyPrice = energy * _this->ToU[hour][TOU_BUY];
			res = _this->ToU[hour][TOU_BUY];
		}
		else {
			*energyPrice = energy * _this->ToU[hour][TOU_SELL];
			res = _this->ToU[hour][TOU_SELL];
		}
		Debug_out(DEBUG, "EBR_energyPrice (ToU): energy = %lg, tariff = %lg, price = %lg\n", energy, res, *energyPrice);
	}
	else if (_this->tariff == TARIFF_FLAT) {
		if (energy >= 0) {
			*energyPrice = energy * _this->FLAT[TOU_BUY];
			res = _this->FLAT[TOU_BUY];
		}
		else {
			*energyPrice = energy * _this->FLAT[TOU_SELL];
			res = _this->FLAT[TOU_SELL];
		}
		Debug_out(DEBUG, "EBR_energyPrice (FLAT): energy = %lg, tariff = %lg, price = %lg\n", energy, res, *energyPrice);
	}
	else if (_this->tariff == TARIFF_SUBST) {
		*energyPrice = energy * _this->SUBSToU[0];
		res = _this->SUBSToU[0];
		Debug_out(DEBUG, "EBR_energyPrice (Substation ToU): energy = %lg, tariff = %lg, price = %lg\n", energy, res, *energyPrice);
	}

	return res;
}



void EBR_readToUTariffs(EBR *_this, const char * tariffsToUFile)
{
	Debug_assert(DEBUG, _this != NULL, "_this == NULL\n");
	Debug_assert(DEBUG, _this->tariff == TARIFF_TOU, "_this->tariff != TARIFF_TOU\n");

	static char line[MAX_BUF_LEN];

	char * token;
	FILE * tariffsFile_fd;
	int i;

	tariffsFile_fd = fopen(tariffsToUFile, "r");
	if (!tariffsFile_fd) {
		perror("Problems opening tariffs file");
		exit(1);
	}
	if (!fgets(line, MAX_BUF_LEN, tariffsFile_fd)) {
		fprintf(stderr, "Problems reading from %s\n", tariffsToUFile);
		exit(1);
	}
	if (line[strlen(line)-1] == '\n') {
		line[strlen(line)-1] = '\0';
	}
	Debug_out(DEBUG, "Price policies first line : %s\n", line);

	for (i = 0; i < 24; i++) {
		int hour;

		if (!fgets(line, MAX_BUF_LEN, tariffsFile_fd)) {
			fprintf(stderr, "Problems reading from %s\n", tariffsToUFile);
			exit(1);
		}
		if (line[strlen(line)-1] == '\n') {
			line[strlen(line)-1] = '\0';
		}
		/* hour */
		token = strtok(line, ",");
		if (!token) {
			perror("ERROR in tokenize");
			exit(1);
		}
		if (StringUtils_toInt(token, &(hour)) != 0) {
			Debug_out(DEBUG, "Problems reading flatBuy, value %s not accepted\n", token);
			exit(1);
		}
		if (hour != i) {
			Debug_out(DEBUG, "Format of file %s not valid: invalid hour %s\n", tariffsToUFile, token);
			exit(1);
		}
		/* buy */
		token = strtok(NULL, ",");
		if (!token) {
			perror("ERROR in tokenize");
			exit(1);
		}
		if (StringUtils_toDouble(token, &(_this->ToU[i][TOU_BUY])) != 0) {
			Debug_out(DEBUG, "Problems reading ToU Buy tariff for hour %d, value %s not accepted\n", i, token);
			exit(1);
		}
		/* sell */
		token = strtok(NULL, ",");
		if (!token) {
			perror("ERROR in tokenize");
			exit(1);
		}
		if (StringUtils_toDouble(token, &(_this->ToU[i][TOU_SELL])) != 0) {
			Debug_out(DEBUG, "Problems reading ToU Sell tariff for hour %d, value %s not accepted\n", i, token);
			exit(1);
		}
	}
	fclose(tariffsFile_fd);
}



int EBR_stepPricePolicy(EBR *_this)
{
	Debug_assert(DEBUG, _this != NULL, "_this == NULL\n");
	Debug_assert(DEBUG, _this->tariff == TARIFF_IBR, "_this->tariff != TARIFF_IBR\n");
	static char line[MAX_BUF_LEN];
	char *token = NULL;
	int h = 0;

	if (_this->pricePolicyFileCurrentLine > 1) {
		h = _this->forecastHours - 1;
	}

	/* read _this->timeslotsAhead slots ahead (first time fills all _this->pricePolicy */
	do {
		/* shift current _this->pricePolicy[] vectors towards left */
		double min, max;
		int t;
		for (t = 0; t < _this->timeslotsAhead - _this->timeslotsInHour; t++) {
			_this->pricePolicy[PRICEPOLICY_MIN][t] = _this->pricePolicy[PRICEPOLICY_MIN][t + _this->timeslotsInHour];
			_this->pricePolicy[PRICEPOLICY_MAX][t] = _this->pricePolicy[PRICEPOLICY_MAX][t + _this->timeslotsInHour];
		}

		if (!fgets(line, MAX_BUF_LEN, _this->pricePolicyFile_fd)) {
			Debug_out(DEBUG, "Problems reading from %s\n", _this->pricePolicyFile);
			return -1;
		}
		if (line[strlen(line)-1] == '\n') {
			line[strlen(line)-1] = '\0';
		}
		Debug_out(DEBUG, "Price policies line : %s\n", line);

		/* tokenize and put pricepolicies into last _this->pricePolicy */
		/* starting */
		token = strtok(line, ",");
		if (!token) {
			perror("ERROR in tokenize");
			return -1;
		}
		/* ending */
		token = strtok(NULL, ",");
		if (!token) {
			perror("ERROR in tokenize");
			return -1;
		}
		/* profile min */
		token = strtok(NULL, ",");
		if (!token) {
			perror("ERROR in tokenize");
			return -1;
		}
		if (StringUtils_toDouble(token, &min) != 0) {
			Debug_out(DEBUG, "Problems reading profile min, value %s not accepted\n", token);
			return -1;
		}
		/* profile max */
		token = strtok(NULL, ",");
		if (!token) {
			Debug_out(DEBUG, "ERROR in tokenize\n");
			perror("");
			return -1;
		}
		if (StringUtils_toDouble(token, &max) != 0) {
			Debug_out(DEBUG, "Problems reading profile max, value %s not accepted\n", token);
			return -1;
		}

		/* fill last hour */
		while (t < _this->timeslotsAhead) {
			_this->pricePolicy[PRICEPOLICY_MIN][t] = min;
			_this->pricePolicy[PRICEPOLICY_MAX][t] = max;
			t++;
		}

		_this->pricePolicyFileCurrentLine = _this->pricePolicyFileCurrentLine + 1;

		h++;

	} while (h < _this->forecastHours);
	
	return 0;
}

void EBR_dumpPricePolicy(EBR *_this)
{
	Debug_assert(DEBUG, _this != NULL, "_this == NULL\n");
	Debug_assert(DEBUG, _this->tariff == TARIFF_IBR, "_this->tariff != TARIFF_IBR\n");

	int i;
	printf("For next %d timeslots of length %d minutes\nMIN:", _this->timeslotsAhead, _this->timeslotLength);
	for (i = 0; i < _this->timeslotsAhead; i++) {
		printf("\t%lg", _this->pricePolicy[PRICEPOLICY_MIN][i]);
	}
	printf("\nMAX:");
	for (i = 0; i < _this->timeslotsAhead; i++) {
		printf("\t%lg", _this->pricePolicy[PRICEPOLICY_MAX][i]);
	}
	printf("\n");
	fflush(NULL);
}



int EBR_stepSubstTariff(EBR *_this)
{
	Debug_assert(DEBUG, _this != NULL, "_this == NULL\n");
	Debug_assert(DEBUG, _this->tariff == TARIFF_SUBST, "_this->tariff != TARIFF_SUBST\n");

	static char line[MAX_BUF_LEN];
	int res = 0;
	char * token;
	int t;
	int h = TIMESLOTS_AHEAD - 1;
	double cost;

	if (!(_this->tariffsSubstFile_started)) {
		h = 0;
		_this->tariffsSubstFile_started = 1;
	}

	do {

		/* shift already stored tariffs */
		for (t = 0; t < TIMESLOTS_AHEAD-1; t++) {
			_this->SUBSToU[t] = _this->SUBSToU[t+1];
		}

		/* read new one, stop if no more lines to read */
		if (!fgets(line, MAX_BUF_LEN, _this->tariffsSubstFile_fd)) {
			fprintf(stderr, "No more lines to read from %s\n", _this->tariffsSubstFile);
			return -1;
		}
		if (line[strlen(line)-1] == '\n') {
			line[strlen(line)-1] = '\0';
		}
		Debug_out(DEBUG, "Retrieved Substation Tariff file line : %s\n", line);

		/* starting */
		token = strtok(line, ",");
		if (!token) {
			perror("ERROR in tokenize");
			return -1;
		}
		/* ending */
		token = strtok(NULL, ",");
		if (!token) {
			perror("ERROR in tokenize");
			return -1;
		}
		/* tariff */
		token = strtok(NULL, ",");
		if (!token) {
			perror("ERROR in tokenize");
			return -1;
		}
		if (StringUtils_toDouble(token, &cost) != 0) {
			Debug_out(DEBUG, "Problems reading Substation Tariff, value %s not accepted\n", token);
			return -1;
		}

		/* input cost is in EUR/MWh while we expect to have EUR/kWh */
		_this->SUBSToU[TIMESLOTS_AHEAD-1] = cost/1000;

		h++;

	} while (h < TIMESLOTS_AHEAD);

	return 0;
}


void EBR_dumpSubstTariff(EBR *_this)
{
	Debug_assert(DEBUG, _this != NULL, "_this == NULL\n");
	Debug_assert(DEBUG, _this->tariff == TARIFF_SUBST, "_this->tariff != TARIFF_SUBST\n");

	int i;
	printf("SUBST ToU (length = %d): ", TIMESLOTS_AHEAD);
	for (i = 0; i < TIMESLOTS_AHEAD; i++) {
		printf("\t%lg", _this->SUBSToU[i]);
	}
	printf("\n");
	fflush(NULL);
}

