#ifndef __HASH_SET__
#define __HASH_SET__

#include "HashTable.h"
#include <stdio.h>
#include <stdlib.h>

/**
Hash-set
**/

struct HashSet;
typedef struct HashSet HashSet;


HashSet* HashSet_new(size_t keySize);

int HashSet_contains(const HashSet* const _this, const void* const key);

void HashSet_add(HashSet* const _this, const void* const key);

void HashSet_remove(HashSet* const _this, const void* const key);

unsigned long HashSet_size(const HashSet* const _this);

void HashSet_free(HashSet** const _this);


/** Returns the number of bytes occupied by a single entry of this hash-set (memory for the key included.
	In other words, it returns the number of bytes that a call fo HashTable_clear() would free-up.
*/
size_t HashSet_Entry_sizeof(const HashSet* const _this);




// HashSet Iterator
struct HashSetIterator;
typedef struct HashSetIterator HashSetIterator;

/** Create an Iterator for t_his HashSet */
HashSetIterator* HashSetIterator_new(const HashSet* const _this);

/** Return 1 if a call to HashSetIterator_next(t_his) will return an entry, 0 otherwise */
int HashSetIterator_hasNext(const HashSetIterator* const _this);

/** Return the next element of the HashSet which 't_his' is iterating on, 
	NULL if no more elements exist */
const void* HashSetIterator_next(HashSetIterator* const _this);
	
/** Free memory used by t_his iterator. Also sets *t_his to NULL. */
void HashSetIterator_free(HashSetIterator** const _this);
	
	
#endif



