#ifndef __PROPERTIES__
#define __PROPERTIES__

#include <HashTable.h>

typedef struct Properties Properties;

typedef struct Properties_Content Properties_Content;
typedef enum { INT, LONG, ULONG, DOUBLE, BOOLEAN, STRING, PROPERTIES } Properties_Content_Type;

// PropertyContent methods
Properties_Content_Type Properties_Content_type(const Properties_Content* const _this);
const void* const Properties_Content_value(const Properties_Content* const _this);
const char* const Properties_Content_Type_toString(Properties_Content_Type type);
int Properties_Content_Value_fprint(const Properties_Content* const _this, FILE* file);

// Constructors and destructor
Properties* Properties_new(int keyMaxLen, int valueMaxLen);
Properties* Properties_newFromFile(const char* const fname, int keyMaxLen, int valueMaxLen);
void Properties_free(Properties** _thisP);
Properties* Properties_clone(const Properties* const _this);

// Empties _this
void Properties_clear(Properties* _this);

// Setter methods (they redefine already defined properties)
void Properties_setDouble(const Properties* const _this, const char* const property, double value);
void Properties_setInt(const Properties* const _this, const char* const property, int value);
void Properties_setULong(const Properties* const _this, const char* const property, unsigned long value);
void Properties_setLong(const Properties* const _this, const char* const property, long value);
void Properties_setBoolean(const Properties* const _this, const char* const property, int value);

//! String value can be NULL
void Properties_setString(const Properties* const _this, const char* const property, const char* const value);
void Properties_setNestedProperties(const Properties* const _this, const char* const property, const Properties* const value);

void Properties_set(const Properties* const _this, const char* const property, Properties_Content_Type type, const void* value);

// Returns 0 in case of success, -1 otherwise
int Properties_setFromString(const Properties* const _this, const char* const property, Properties_Content_Type type, const char* const value_string);


// Prints out a textual representation of _this to f
int Properties_fprint(const Properties* const _this, FILE* f);

// Fills result with a string representation of _this (up to length chars are written to result).
//void Properties_toString(const Properties* const _this, char* result, int length);

// Saves _this to file 'filePath'
int Properties_save(const Properties* const _this, char* filePath);

// Getter methods: they fill-in 'property' value to *value
int Properties_getInt(const Properties* const _this, const char* const property, int* value);
int Properties_getULong(const Properties* const _this, const char* const property, unsigned long* value);
int Properties_getLong(const Properties* const _this, const char* const property, long* value);
int Properties_getBoolean(const Properties* const _this, const char* const property, int* value);
int Properties_getDouble(const Properties* const _this, const char* const property, double* value);
int Properties_getString(const Properties* const _this, const char* const property, const char** const value);
int Properties_getNestedProperties(const Properties* const _this, const char* const property, const Properties** const value);

const Properties_Content* const Properties_get(const Properties* const _this, const char* const property);

int Properties_contains(const Properties* const _this, const char* const property);

// Returns type of 'property' in 'this'. Assertion error if 'property' does not exist in 'this'
Properties_Content_Type Properties_type(const Properties* const _this, const char* const property);

// Returns the number of properties in this
unsigned long Properties_size(const Properties* const _this);

typedef HashTable_Entry Properties_Entry; 
typedef struct Properties_Iterator Properties_Iterator;


/** Create an Iterator for this Properties */
Properties_Iterator* Properties_Iterator_new(const Properties* const _this);
void Properties_Iterator_rewind(Properties_Iterator* const _this);

/** Return 1 if a call to Properties_Iterator_next(this) will return an entry, 0 otherwise */
int Properties_Iterator_hasNext(const Properties_Iterator* const _this);

/** Return the next entry of the HashTable which 'this' is iterating on, 
	NULL if no more entries exist */
const Properties_Entry* const Properties_Iterator_next(Properties_Iterator* const _this);

const char* const Properties_Entry_key(const Properties_Entry* const _this);
const char* const Properties_Entry_property(const Properties_Entry* const _this);

const Properties_Content* const Properties_Entry_value(const Properties_Entry* const _this);

void Properties_Iterator_free(Properties_Iterator** thisP);
#endif

