#ifndef __STRING_UTILS__
#define __STRING_UTILS__

#include <Array.h>

int StringUtils_lastIndexOf(const char* str, char c);
char* StringUtils_clone(const char* str);

/* Creates a new string consisting of the substring of 'str' from index 'from' to index 'to' (inclusive).
   Both 'from' and 'to' can be negative: in this case, they count from the end of the string
*/
char* StringUtils_substring(const char* str, int from, int to);

// Modifies 's' by removing white spaces at both ends. Returns a pointer to 's'.
char* StringUtils_trim(char* str);

int StringUtils_strpos(char* s, char c);
int StringUtils_replaceAll(char* s, char* findChars, char* replaceChars);
void StringUtils_tokenize(char* s, char* sep, Array* tokens);

char* StringUtils_rightPad(char* s, int length, char pad);
char* StringUtils_leftPad(char* s, int length, char pad);

int StringUtils_startsWith(const char* const s, const char* const needle);
int StringUtils_endsWith(const char* const s, const char* const needle);

// Fills *result with 's' converted into int. Returns 0 on success, an error code (<0) otherwise
int StringUtils_toInt(const char* const s, int* result);
int StringUtils_toLong(const char* const s, long* result);
int StringUtils_toULong(const char* const s, unsigned long* result);

// Fills *result with 's' converted into double. Returns 0 on success, an error code (<0) otherwise
int StringUtils_toDouble(const char* const s, double* result);

/* Fills *result with 's' converted into int 1 (true) or 0 (false). 
   Returns 0 on success, an error code (<0) otherwise */
int StringUtils_toBoolean(const char* const s, int* result);

int StringUtils_equals(const char* const s, const char* const s2);
int StringUtils_equalsIgnoreCase(const char* const s, const char* const s2);

#endif

