#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include <Debug.h>
#define DEBUG_FU "FileUtils"

#include <FileUtils.h>


int FileUtils_copyFile(const char *from, const char *to) {
    int fd_to, fd_from;
    char buf[4096];
    ssize_t nread;
    int saved_errno;

    fd_from = open(from, O_RDONLY);
    if (fd_from < 0) {
		Debug_out(DEBUG_FU, "fd_from == NULL\n");
		return -1;
	}

    fd_to = creat(to, 0666);
    if (fd_to < 0) goto out_error;

    while (nread = read(fd_from, buf, sizeof buf), nread > 0) {
        char *out_ptr = buf;
        ssize_t nwritten;

        do {
            nwritten = write(fd_to, out_ptr, nread);

            if (nwritten >= 0) {
                nread -= nwritten;
                out_ptr += nwritten;
            }
            else if (errno != EINTR) {
                goto out_error;
            }
        } while (nread > 0);
    }

    if (nread == 0) {
        if (close(fd_to) < 0) {
            fd_to = -1;
            goto out_error;
        }
        close(fd_from);

        /* Success! */
        return 0;
    }

  out_error:
    saved_errno = errno;

    close(fd_from);
    if (fd_to >= 0)
        close(fd_to);

    errno = saved_errno;
	Debug_out(DEBUG_FU, "errno = %d\n", errno);
    return -1;
}

