#include <Process.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>

#include <sys/utsname.h>
#include <sys/types.h>
#include <sys/resource.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/ioctl.h>

#include <unistd.h>
#include <assert.h>



#define BUFFER_LEN 1014


struct Process {
	FILE *ioStreams;
	pid_t pid;
};




static double _Process_convert_date_to_seconds(char *);





Process *Process_new(char *program)
{
	int fds[2];
	pid_t pid;
	Process *res = (Process *) calloc(1, sizeof(Process));

	if (socketpair(AF_UNIX, SOCK_STREAM, 0, fds) < 0)
		return NULL;

	switch (pid = vfork()) {
		case -1:    /* Error */
			close(fds[0]);
			close(fds[1]);
			return NULL;
		case 0:     /* child */
			close(fds[0]);
			dup2(fds[1], 0);
			dup2(fds[1], 1);
			close(fds[1]);
			execl("/bin/sh", "sh", "-c", program, NULL);
			_exit(127);
	}
	/* parent */
	close(fds[1]);
	res->ioStreams = fdopen(fds[0], "r+");
	res->pid = pid;
	return res;
}



void Process_dispose(Process **_this)
{
	fclose((*_this)->ioStreams);
	free(*_this);
}



void Process_dump(Process *_this, FILE *stream)
{
	fprintf(stream, "pid = %d\n", _this->pid);
}


long Process_read(Process *_this, char *buffer)
{
    char *buf = (char *) calloc(BUFFER_LEN, sizeof(char));;
    int len = 0;

    sprintf(buf, "");
    while (fgets(buf, BUFFER_LEN, _this->ioStreams)) {
        len += strlen(buf);
        sprintf(buffer, "%s%s", buffer, buf);
    }
    return len;
}


long Process_write(Process *_this, char *buffer)
{
    long res = 0;
    return res;
}



long Process_RAM_Usage(Process *_this)
{
    static int len;
    static long res;
    static char buf[BUFFER_LEN];
    static char *cmd;
    static FILE *out;
    static struct utsname name;
    
    /* distinguish among Mac OS (Darwin) and Linux */
    assert(uname(&name) == 0);
    if (strcmp(name.sysname, "Darwin") == 0) {
        len = snprintf(NULL, 0, "ps -p %d -o vsz | tail -1 | awk '{printf(\"%%s\", $1)}'", _this->pid) + 1;
        cmd = (char *) calloc(len, sizeof(char));
        sprintf(cmd, "ps -p %d -o vsz | tail -1 | awk '{printf(\"%%s\", $1)}'", _this->pid);
    }
    else /* if (strcmp(name.sysname, "Linux") == 0) */ {
        len = snprintf(NULL, 0, "cat /proc/%d/stat | awk '{print $23;}'", _this->pid) + 1;
        cmd = (char *) calloc(len, sizeof(char));
        sprintf(cmd, "cat /proc/%d/stat | awk '{print $23;}'", _this->pid);
    }
    out = popen(cmd, "r");
    if (out == NULL) {
        printf("Error retrieving this Process PID=%d memory size", _this->pid);
        exit(1);
    }
    res = strtol(fgets(buf, BUFFER_LEN, out), NULL, 10);
    pclose(out);
    free(cmd);
    return res;
}





double Process_dump_Time_RAM_Usage(Process *_this, const char *string, FILE *outstream)
{
    static char to_be_exec[BUFFER_LEN];
    double ret;
    FILE *stream;
    int count = 0;
    char *tmp, *tmp2;
    struct utsname name;
    
    /* distinguish among Mac OS (Darwin) and Linux */
    assert(uname(&name) == 0);
    if (strcmp(name.sysname, "Darwin") == 0) {
        sprintf(to_be_exec, "ps -p %d -o utime | tail -1 | awk '{printf(\"%%s\", $1);}'", _this->pid);
        stream = popen(to_be_exec, "r");
        fgets(to_be_exec, BUFFER_LEN, stream);
        pclose(stream);
        ret = _Process_convert_date_to_seconds(to_be_exec);
        if (strcmp(string, "") != 0) {
            long mem;
            sprintf(to_be_exec, "ps -p %d -o vsz | tail -1 | awk '{printf(\"%%s\", $1)}'", _this->pid);
            stream = popen(to_be_exec, "r");
            fgets(to_be_exec, BUFFER_LEN, stream);
            pclose(stream);
            mem = atol(to_be_exec);
            fprintf(outstream, "[%d] Execution time and memory used at %s: %lg seconds, %ld bytes of memory usage\n", _this->pid, string, ret/100, mem);
            fflush(NULL);
        }
    }
    else if (strcmp(name.sysname, "Linux") == 0) {
        if (strcmp(string, "") != 0) {
            sprintf(to_be_exec, "cat /proc/%d/stat | awk '{printf(\"%%e seconds, %%s bytes of memory usage and \", $14/100, $23)}'; \
                    cat /proc/%d/status | grep VmPeak | awk '{printf(\"%%s kB of memory peak\\n\", $2)}'", _this->pid, _this->pid);
            fprintf(outstream, "[%d] Execution time and memory used at %s: ", _this->pid, string);
            fflush(NULL);
            system(to_be_exec);
            fflush(NULL);
        }
        sprintf(to_be_exec, "/proc/%d/stat", _this->pid);
        stream = fopen(to_be_exec, "r");
        fgets(to_be_exec, BUFFER_LEN, stream);
        fclose(stream);
        tmp = to_be_exec;
        while (count < 13) {
            tmp = strchr(tmp + 1, ' ');
            count++;
        }
        tmp2 = strchr(tmp + 1, ' ');
        *tmp2 = '\0';
        ret = atof(tmp)/100;
    }
    else {
        /* operating system not supported */
        ret = 0.0;
    }
    return ret;
}



static double _Process_convert_date_to_seconds(char *date)
{
    double ret = 0.0;
    char *tok = (char *)0;
    char *str = date;
    if (strchr(date, '-') != NULL) {
        /* dd-hh:mm.ss */
        tok = strtok(str, "-");
        ret += atol(tok) * (60*60*24);
        str = NULL;
    }
    /* hh:mm.ss */
    tok = strtok(str, ":");
    ret += atol(tok) * (60*60);
    /* mm.ss */
    tok = strtok(NULL, ".");
    ret += atol(tok) * 60;
    /* ss */
    tok = strtok(NULL, ".");
    ret += atol(tok);
    return ret;
}

