#include <Debug.h>
#include <TimeUtils.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>


#define DEBUG "TimeUtils"


struct TimeUtils {
	int year;
	int month;
	int day;
	int hour;
	int minute;
	int second;
};


static void _TimeUtils_normilise(TimeUtils **);
static int _TimeUtils_isLeapYear(int year);



TimeUtils *TimeUtils_new(const char *_date)
{
	int scanfout;
	TimeUtils *res = (TimeUtils *) calloc(1, sizeof(TimeUtils));

	scanfout = sscanf(_date, "%4d-%2d-%2d %2d:%2d:%2d",
					  &res->year, &res->month, &res->day,
					  &res->hour, &res->minute, &res->second);

	Debug_assert(DEBUG, scanfout == 6, "TimeUtils format must be \"YYYY-MM-DD HH:MM:SS\" with HH in 0:23\n");
	Debug_assert(DEBUG, 1 <= res->month && res->month <= 12, "Month %d out of range 1..12\n", res->month);
	Debug_assert(DEBUG, 1 <= res->day && res->day <= 31, "Day %d out of range 1..31\n", res->day);
	Debug_assert(DEBUG, 0 <= res->hour && res->hour <= 23, "Hour %d out of range 0..23\n", res->hour);
	Debug_assert(DEBUG, 0 <= res->minute && res->minute <= 59, "Minute %d out of range 0..59\n", res->minute);
	Debug_assert(DEBUG, 0 <= res->second && res->second <= 59, "Second %d out of range 0..59\n", res->second);
	return res;
}

void TimeUtils_dispose(TimeUtils **_this)
{
	Debug_assert(DEBUG, _this != NULL, "_this == NULL\n");
	free(*_this);
}

TimeUtils *TimeUtils_copy(TimeUtils *_this)
{
	TimeUtils *new = NULL;
	if (_this) {
		new = (TimeUtils *) calloc(1, sizeof(TimeUtils));
		new->year = _this->year;
		new->month = _this->month;
		new->day = _this->day;
		new->hour = _this->hour;
		new->minute = _this->minute;
		new->second = _this->second;
	}
	return new;
}

int TimeUtils_eq(TimeUtils *a, TimeUtils *b)
{
	Debug_assert(DEBUG, a != NULL, "a == NULL\n");
	Debug_assert(DEBUG, b != NULL, "b == NULL\n");
	return ((a->second == b->second) && (a->minute == b->minute) && (a->hour == b->hour) && (a->day == b->day) && (a->month == b->month) && (a->year == b->year));
}

int TimeUtils_lt(TimeUtils *a, TimeUtils *b)
{
	Debug_assert(DEBUG, a != NULL, "a == NULL\n");
	Debug_assert(DEBUG, b != NULL, "b == NULL\n");
	return ((a->year < b->year) ||
			(a->year == b->year && a->month < b->month) ||
			(a->year == b->year && a->month == b->month && a->day < b->day) ||
			(a->year == b->year && a->month == b->month && a->day == b->day && a->hour < b->hour) ||
			(a->year == b->year && a->month == b->month && a->day == b->day && a->hour == b->hour && a->minute < b->minute) ||
			(a->year == b->year && a->month == b->month && a->day == b->day && a->hour == b->hour && a->minute == b->minute && a->second < b->second));
}

void TimeUtils_addSeconds(TimeUtils **_this, int seconds)
{
	(*_this)->second += seconds;
	_TimeUtils_normilise(_this);
	return;
}

void TimeUtils_dump(FILE *stream, TimeUtils *_this)
{
	if (_this) {
		fprintf(stream, "%d-%02d-%02d %02d:%02d:%02d\n",
				_this->year, _this->month, _this->day,
				_this->hour, _this->minute, _this->second);
	}
	else
		fprintf(stream, "Empty Time\n");
}

char * TimeUtils_toString(TimeUtils *_this)
{
	char *res = (char *) calloc(22, sizeof(char));
	sprintf(res, "%d-%02d-%02d %02d:%02d:%02d",
			_this->year, _this->month, _this->day,
			_this->hour, _this->minute, _this->second);
	return res;
}




static void _TimeUtils_normilise(TimeUtils **_this)
{
	int daysPerMonth[12] = {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
	int i;
	int moreDays = 0;

	Debug_perform(DEBUG, TimeUtils_dump(stdout, *_this));

	if ((*_this)->second >= 60) {
		int moreMins = 0;
		while ((*_this)->second >= 60) {
			(*_this)->second -= 60;
			moreMins++;
		}
		(*_this)->minute += moreMins;
	}

	Debug_perform(DEBUG, TimeUtils_dump(stdout, *_this));

	if ((*_this)->minute >= 60) {
		int moreHours = 0;
		while ((*_this)->minute >= 60) {
			(*_this)->minute -= 60;
			moreHours++;
		}
		(*_this)->hour += moreHours;
	}

	Debug_perform(DEBUG, TimeUtils_dump(stdout, *_this));

	if ((*_this)->hour >= 24) {
		moreDays = 0;
		while ((*_this)->hour >= 24) {
			(*_this)->hour -= 24;
			moreDays++;
		}
	}

	Debug_perform(DEBUG, TimeUtils_dump(stdout, *_this));

	Debug_out(DEBUG, "moreDays = %d\n", moreDays);

	for (i = 0; i < moreDays; i++) {
		((*_this)->day)++;
		Debug_out(DEBUG, "(*_this)->day = %d\n", (*_this)->day);
		Debug_out(DEBUG, "daysPerMonth[(*_this)->month-1] = %d\n", daysPerMonth[(*_this)->month-1]);
		Debug_out(DEBUG, "_TimeUtils_isLeapYear((*_this)->year) = %d\n", _TimeUtils_isLeapYear((*_this)->year));
		if (((*_this)->day > daysPerMonth[(*_this)->month-1]) ||
			((*_this)->month == 2 && (*_this)->day == 29 && !_TimeUtils_isLeapYear((*_this)->year))) {
			(*_this)->day = 1;
			((*_this)->month)++;
			if ((*_this)->month == 13) {
				(*_this)->month = 1;
				((*_this)->year)++;
			}
		}
	}

	Debug_perform(DEBUG, TimeUtils_dump(stdout, *_this));
}

// this follows Gregorian rules
static int _TimeUtils_isLeapYear(int year)
{
	return (year % 400 == 0 || (year % 100 != 0 && year % 4 == 0));
}
