#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <HashTable.h>
#include <StringUtils.h>

#include <Debug.h>

#define KEY_LEN 101

typedef struct {
	char name[KEY_LEN];
	time_t start;
	time_t end;	
} Timer;

static HashTable* timers = {0};
static char fullLengthName[KEY_LEN] = {0};

void _Timer_createFullLengthName(const char* const timerName) {
	strcpy(fullLengthName, timerName);
	StringUtils_rightPad(fullLengthName, KEY_LEN, '\0');
}

Timer* _Timer_new(const char* const name) {
	Timer* new = calloc(1, sizeof(Timer));
	_Timer_createFullLengthName(name);
	strcpy(new->name, fullLengthName);
	new->start = 0;
	new->end = 0;
	return new;
}

void _Timer_free(Timer** thisP) {
	Debug_assert(DEBUG_ALL, thisP != NULL, "thisP == NULL\n");
	Timer* this = *thisP;
	free(this);
	*thisP = NULL;
}




void Timer_start(const char* const name) {
	if (timers == NULL) {
		timers = HashTable_new(sizeof(char)*(KEY_LEN+1));
	}
	_Timer_createFullLengthName(name);
	Timer* timer = NULL;
	HashTable_get(timers, fullLengthName, (void**)&timer);
	if (timer == NULL) {
		// Create a new timer
		timer = _Timer_new(name);
		HashTable_put(timers, fullLengthName, timer);
	}
	timer->start = time(NULL);
	timer->end = 0;
// 	Debug_out(DEBUG_ALWAYS, "Timer_start(%s): timer started\n", timer->name);
	printf("%s: TIMER STARTED\n", timer->name);
}

void Timer_end(const char* const name) {
	_Timer_createFullLengthName(name);
	Timer* timer = NULL;
	HashTable_get(timers, fullLengthName, (void**)&timer);
	if (timer == NULL) {
		Debug_out(DEBUG_ALWAYS, "Timer_end(%s): timer has not been created\n", name);
	}
	timer->end = time(NULL);
	double timeDiff = difftime(timer->end, timer->start);
	HashTable_clear(timers, fullLengthName);
// 	Debug_out(DEBUG_ALWAYS, "Timer_end(%s): time elapsed = %lf seconds\n", timer->name, timeDiff);
	printf("%s: TIME ELAPSED = %lf SECONDS\n", timer->name, timeDiff);
	_Timer_free(&timer);

	if (HashTable_size(timers) == 0) {
		HashTable_free(&timers);
	}
}

