

/***
Not robust, as pointers to internal elements of the array could be dandling in case
Array triggers a realloc
***/

#include <stdio.h>
#include <stdlib.h>

#include <Debug.h>

#include <Array.h>
#include <HashTable.h>
#include <BasicTypeValue.h>

#define DEBUG "BasicTypeCache"

#include <BasicTypeCache.h>

struct BasicTypeCache {
	HashTable* map;
	Array* memory;
};

BasicTypeCache* BasicTypeCache_new(size_t keyLength, int capacity, int capIncrement) {
	BasicTypeCache* new = calloc(1, sizeof(BasicTypeCache));
	Debug_assert(DEBUG_ALWAYS, new != NULL, "new == NULL\n");
	new->map = HashTable_new(keyLength);
	new->memory = Array_new(capacity, sizeof(BasicTypeValue), capIncrement, NULL, NULL);
	return new;
}

void BasicTypeCache_free(BasicTypeCache** thisP) {
	Debug_assert(DEBUG_ALWAYS, thisP != NULL, "thisP == NULL\n");
	BasicTypeCache* this = *thisP;
	Array_free(&this->memory);
	HashTable_free(&this->map);
	free(this);
	*thisP = NULL;
}

BasicTypeValue* BasicTypeCache_get(const BasicTypeCache* const this, void* key) {
	BasicTypeValue* result = NULL;
	HashTable_get(this->map, key, (void**)&result);
	return result;
}

void BasicTypeCache_put(BasicTypeCache* const this, void* key, BasicType type, void* value_p) {
	BasicTypeValue v = {0};
	BasicTypeValue_set(&v, type, value_p);
	Array_add(this->memory, &v);
	const void* cache_value_ptr = Array_elemPtr(this->memory, Array_length(this->memory)-1);
	HashTable_put(this->map, key, (void*)cache_value_ptr);
	
	Debug_assert(DEBUG, BasicTypeCache_get(this, key) != NULL,
		"Error in put(): _get() returned NULL\n");
		
	Debug_assert(DEBUG, BasicTypeCache_get(this, key) == cache_value_ptr, 
		"Error in put(): _get() returned %p, cache_value_ptr == %p\n",
		BasicTypeCache_get(this, key), cache_value_ptr
	);
}

void BasicTypeCache_clear(BasicTypeCache* const this) {
	Debug_assert(DEBUG, this != NULL, "this == NULL\n");
	HashTableIterator* it = HashTableIterator_new(this->map);
	HashTable_Entry* e = NULL;
	while ( (e = HashTableIterator_next(it)) != NULL ) {
		HashTable_clear(this->map, HashTable_Entry_key(e));
	}
	HashTableIterator_free(&it);
	Array_clear(this->memory);
}

