#include <stdio.h>
#include <stdlib.h>

#include <Array.h>
#include <Debug.h>

#define DEBUG "TestArray"


int testIntArray() {	
	Array* a = Array_newInt(2, 2);
	int value = 1;

	Array_add(a, &value);	
	value = 2;
	Array_add(a, &value);
	Array_fprint(a, stderr);
	
	value = 3;
	Array_add(a, &value);
	
	Array_fprint(a, stderr);
	
	value = 5;
	Array_set(a, 1, &value);
	Array_fprint(a, stderr);


	Array* clone = Array_clone(a);
	fprintf(stderr, "Clone array: ");
	Array_fprint(clone, stderr);
	fprintf(stderr, "Clone equals array? %d\n", Array_equals(a, clone));

	fprintf(stderr, "Swapping elements 1 and 2 in clone. Now: ");
	Array_swap(clone, 1, 2);
	Array_fprint(clone, stderr);
	
	Array_free(&a);
	Array_free(&clone);
	return 0;	
}


typedef struct {
	int id;
	const char* name;
} MyStruct;

MyStruct* MyStruct_new(int id, const char* const name) {
	MyStruct* new = calloc(1, sizeof(MyStruct));
	new->id = id;
	new->name = name;
	return new;
}
void MyStruct_free(MyStruct** const thisP) {
	MyStruct* this = *thisP;
	free(this);
	*thisP = NULL;
}

void MyStruct_fprint(const void* const v, FILE* f) {	
	MyStruct* p = (MyStruct*)v;
	if (p == NULL) {
		fprintf(f, "(%p)", (void*)p);
	} else {
		//fprintf(f, "vpp=%p --> p=%p --> [id: %d, name: '%s']", (void*)vpp, (void*)p, p->id, p->name);
		fprintf(f, "[id: %d, name: '%s']", p->id, p->name);
	}
}
void MyStruct_fprint_pp(const void* const vpp, FILE* f) {
	Debug_assert(DEBUG, vpp != NULL, "vpp == NULL\n");
	MyStruct** pp = (MyStruct**)vpp;
	MyStruct_fprint(*pp, f);
}	


int testPtrArray() {	
	Array* a = Array_new(2, sizeof(MyStruct*), 2, MyStruct_fprint_pp, NULL);
	MyStruct* p = MyStruct_new(1, "ciao");
	//fprintf(stderr, "Adding to array: &p = %p; p=%p\n", (void*)&p, (void*)p); //MyStruct_fprint(stderr, p); fprintf(stderr, "\n");
	Array_add(a, &p);

	Array_fprint(a, stderr);

	p = MyStruct_new(2, "hello");
	//fprintf(stderr, "Adding to array: &p = %p; p=%p\n", (void*)&p, (void*)p); //MyStruct_fprint(stderr, p);	fprintf(stderr, "\n");
	Array_add(a, &p);

	Array_fprint(a, stderr);	
	
	//fprintf(stderr, "\n\nCreating 3rd element\n");
	p = MyStruct_new(3, "hola");
	//fprintf(stderr, "&p = %p; p=%p\n", (void*)&p, (void*)p); 
	//fprintf(stderr, "&pp = %p; pp=", (void*)&pp); MyStruct_fprint(stderr, pp);	fprintf(stderr, "\n");	
	
	Array_add(a, &p);
	
	Array_fprint(a, stderr);
	
	p = MyStruct_new(1, "CIAO");
	Array_set(a, 0, &p);

	Array_fprint(a, stderr);
	
	fprintf(stderr, "=== Start testing Array_elemPtr() ===\n");
	MyStruct** c_array = (MyStruct**)Array_as_C_array(a);
	int len = Array_length(a);
	fprintf(stderr, "%d-th element of the C array: %p\n", len-1, (void*)&c_array[len-1]);
	MyStruct_fprint(c_array[len-1], stderr );

	fprintf(stderr, "\n%d-th element of the Array: %p\n", len-1, Array_elemPtr(a, len-1));
	MyStruct_fprint_pp(Array_elemPtr(a, len-1), stderr );
	fprintf(stderr, "=== \nEnd testing Array_elemPtr() : result = '%s' ===\n",
		&c_array[len-1] == Array_elemPtr(a, len-1) ? "PASS" : "FAIL");	
	
	
	Array_free(&a);
	return 0;	
}



int testStrArray() {	
	Array* a = Array_newString(2, 2);
	char* s = "ciao";
	Array_add(a, &s);

	Array_fprint(a, stderr);

	s = "hello";
	Array_add(a, &s);

	Array_fprint(a, stderr);	
	
	s = "hola";
	Array_add(a, &s);
	
	Array_fprint(a, stderr);
	
	s = "CIAO";
	Array_set(a, 0, &s);

	Array_fprint(a, stderr);
	
	Array* clone = Array_clone(a);
	fprintf(stderr, "Clone array: ");
	Array_fprint(clone, stderr);
	fprintf(stderr, "Clone equals array? %d\n", Array_equals(a, clone));	
	
	
	Array_free(&a);
	Array_free(&clone);	
	return 0;	
}

int testStrQueue() {	
	fprintf(stderr, "\n\n TEST ARRAY AS A QUEUE\n\n");
	Array* a = Array_newString(2, 1);

	char* s = "ciao";
	Array_enqueue(a, &s);
	fprintf(stderr, " - enqueue(%s)\n", s);
	Array_fprint(a, stderr);

	s = "hello";
	Array_enqueue(a, &s);
	fprintf(stderr, " - enqueue(%s)\n", s);	
	Array_fprint(a, stderr);	

	Array_dequeue(a, &s);
	fprintf(stderr, " - dequeue() = %s\n", s);
	Array_fprint(a, stderr);

	Array_dequeue(a, &s);
	fprintf(stderr, " - dequeue() = %s\n", s);
	Array_fprint(a, stderr);

	
	s = "hola";
	Array_enqueue(a, &s);
	fprintf(stderr, " - enqueue(%s)\n", s);	
	
	Array_fprint(a, stderr);
	
	s = "CIAO";
	Array_enqueue(a, &s);	
	fprintf(stderr, " - enqueue(%s)\n", s);			
	Array_fprint(a, stderr);

	Array_dequeue(a, &s);
	fprintf(stderr, " - dequeue() = %s\n", s);
	Array_fprint(a, stderr);
	
	Array_dequeue(a, &s);
	fprintf(stderr, " - dequeue() = %s\n", s);
	Array_fprint(a, stderr);
		
	s = "Hullo";
	Array_enqueue(a, &s);	
	fprintf(stderr, " - enqueue(%s)\n", s);			
	Array_fprint(a, stderr);
		
	s = "Bye";
	Array_enqueue(a, &s);	
	fprintf(stderr, " - enqueue(%s)\n", s);			
	Array_fprint(a, stderr);
		

	/*fprintf(stderr, " - dequeuing from an empty array...\n");
	Array_dequeue(a, &s);
	fprintf(stderr, " - dequeue() = %s\n", s);				
	Array_fprint(stderr, a);*/
	
	Array_free(&a);
	return 0;	
}

int main() {
	testIntArray();
	testPtrArray();
	testStrArray();
	testStrQueue();
	return 0;
}
