#include "HashTable.h"
#include <stdio.h>
#include <stdlib.h>


typedef struct {
	int x;
	int y;
} HTableTestKey;
HTableTestKey* HTableTestKey_new(int _x, int _y) {
	HTableTestKey* new = (HTableTestKey*)calloc(1, sizeof(HTableTestKey));
	new->x = _x;
	new->y = _y;
	return new;
}

char* HTableTestKey_toString(const void* this, char* str) {
	HTableTestKey* p = (HTableTestKey*)this;
	sprintf(str, "(%d, %d)", p->x, p->y);
	return str;
}

typedef struct {
	int x;
	int y;
	int z;
} HTableTestValue;

HTableTestValue* HTableTestValue_new(int _x, int _y, int _z) {
	HTableTestValue* new = (HTableTestValue*)calloc(1, sizeof(HTableTestValue));
	new->x = _x;
	new->y = _y;
	new->z = _z;
	return new;
}

char* HTableTestValue_toString(const void* this, char* str) {
	HTableTestValue* p = (HTableTestValue*)this;	
	sprintf(str, "(%d, %d, %d)", p->x, p->y, p->z);
	return str;
}



void HashTableStructTest() {
	int min = 1;
	int max = 10;
	HashTable* hTable = HashTable_new(sizeof(HTableTestKey));
	int i=0, j=0;

	printf("Build a test hTable (forall i,j : (i,j)--> (i, j, j+1)):\n");
	
	HTableTestKey* keyP = NULL;	
	HTableTestValue* valueP = NULL;

	
	for(i=min; i<=max; i++) {
		for(j=min; j<=max; j++) {
			keyP = (HTableTestKey*)HTableTestKey_new(i,j);
			valueP = (HTableTestValue*)HTableTestValue_new(i,j,j+1);	
			HashTable_put(hTable, keyP, valueP);
		}
	}
	HashTable_print(stderr, hTable, HTableTestKey_toString, HTableTestValue_toString, 10000, 10000);

	printf("Remove entries (i,j) s.t. i%%3 == 0.\n");
	for(i=min; i<=max; i++) {
		if (i%3 != 0) continue;
		for(j=min; j<=max; j++) {
			keyP = (HTableTestKey*)HTableTestKey_new(i,j);
			valueP = HashTable_clear(hTable, keyP);
			if (valueP) free(valueP);
		}
	}
	HashTable_print(stderr, hTable, HTableTestKey_toString, HTableTestValue_toString, 10000, 10000);
	
	HashTable_free(&hTable);
}



char* _ul_toString(const void* p, char* str) {
	unsigned long l = *((unsigned long*)p);
	sprintf(str, "%lu", l);
	return str;
}
void HashTableULTest() {
	unsigned long min = 1;
	unsigned long max = 10;
	HashTable* hTable = HashTable_new(sizeof(unsigned long));
	unsigned long i=0, j=0;

	printf("Build a test hTable (forall i : (i)--> (i+1):\n");
	
	unsigned long* keyP = NULL;	
	unsigned long* valueP = NULL;
		
	for(i=min; i<=max; i++) {
			//keyP = (unsigned long*)calloc(1, sizeof(unsigned long));	
			//*keyP = i;
			valueP = (unsigned long*)calloc(1, sizeof(unsigned long));
			*valueP =i+1;
			HashTable_put(hTable, &i, /*keyP,*/ valueP);
	}
	HashTable_print(stderr, hTable, _ul_toString, _ul_toString, 10000, 10000);

	printf("Remove entries 'i' s.t. i%%3 == 0.\n");
	unsigned long k = 0;
	for(k=min; k<=max; k++) {
		if (k%3 != 0) continue;
		for(j=min; j<=max; j++) {
			//keyP = &k;
			valueP = HashTable_clear(hTable, /*keyP*/ &k);
			if (valueP) free(valueP);
		}
	}
	HashTable_print(stderr, hTable, _ul_toString, _ul_toString, 10000, 10000);
	
	printf("Remove entries 'i' s.t. i%%3 == 1 using an iterator.\n");
	HashTableIterator* it = HashTableIterator_new(hTable);
	HashTable_Entry* e = NULL;
	while ((e = HashTableIterator_next(it)) != NULL) {
		keyP = (unsigned long*)HashTable_Entry_key(e);
		if (*keyP % 3 == 1) {
			printf(" - Removing entry having key '%lu'\n", *keyP);
			valueP = HashTable_clear(hTable, keyP);
			if (valueP) free(valueP);
		}
	}
	HashTableIterator_free(&it);	
	printf("\n\n");
	HashTable_print(stderr, hTable, _ul_toString, _ul_toString, 10000, 10000);	

	// Clear whole htable and free memory allocated for values
	it = HashTableIterator_new(hTable);
	while ((e = HashTableIterator_next(it)) != NULL) {
		keyP = (unsigned long*)HashTable_Entry_key(e);
		valueP = HashTable_clear(hTable, keyP);
		if (valueP) free(valueP);
	}
	HashTableIterator_free(&it);
	
	HashTable_free(&hTable);
}

int main() {
	//HashTableStructTest();
	HashTableULTest();
	return 0;
}
