
#include <stdio.h>

#include <Debug.h>
#include <RndGen.h>
#include <StringUtils.h>

#include <Range_Int.h>

#define DEBUG_THIS "TestRangeInt"

int main(int argc, char** args) {

	Debug_assert(DEBUG_THIS, argc >= 4, "Syntax: %s min max step\n", args[0]);
	int min = 0, max = 0, step = 0; 
	int err = StringUtils_toInt(args[1], &min);
	Debug_assert(DEBUG_THIS, err == 0, "Error while parsing '%s'\n", args[1]);
	err = StringUtils_toInt(args[2], &max);
	Debug_assert(DEBUG_THIS, err == 0, "Error while parsing '%s'\n", args[2]);	
	err = StringUtils_toInt(args[3], &step);
	Debug_assert(DEBUG_THIS, err == 0, "Error while parsing '%s'\n", args[3]);
	
	Range_Int* ri = Range_Int_new(min, max, step);
	fprintf(stderr, "Range_Int has %d values\n", Range_Int_size(ri));
	
	
	fprintf(stderr, " === START TEST: _value() ====\n");		
	int value = 0;
	for(int index = 0; index < Range_Int_size(ri); index++) {
		err = Range_Int_value(ri, index, &value);
		fprintf(stderr, " - [%d] --> ", index);
		if (err) {
			fprintf(stderr, "error %d\n", err);
		} else {
			fprintf(stderr, "%d\n", value);
		}
	}
	fprintf(stderr, " === END TEST: _value() ====\n");			
	
	fprintf(stderr, " === START TEST: _index() ====\n");		
	int index = 0;
	for(value = Range_Int_min(ri); value <= Range_Int_max(ri); value++) {
		err = Range_Int_index(ri, value, &index);
		fprintf(stderr, " - %d --> ", value);
		if (err) {
			fprintf(stderr, "error %d\n", err);
		} else {
			fprintf(stderr, "[%d]\n", index);
		}
	}
	fprintf(stderr, " === END TEST: _index() ====\n");			
	
	
	
	Range_Int_Iterator* it = Range_Int_Iterator_new(ri);
	value = 0;
	
	fprintf(stderr, " === SEQUENTIAL SCAN TEST ====\n");
	for(int round = 1; round <= 2; round++) {
		Debug_out(DEBUG_THIS, "==== START ROUND %d ======\n", round);
		Range_Int_Iterator_rewind(it);
		fprintf(stderr, "Going forward\n");		
		while (Range_Int_Iterator_hasNext(it)) {
			err = Range_Int_Iterator_next(it, &value);
			Debug_assert(DEBUG_THIS, !err, "err = %d\n", err);
			Debug_out(DEBUG_THIS, " - %d\n", value);
		}
		
		fprintf(stderr, "Going backward\n");
		while (Range_Int_Iterator_hasPrev(it)) {
			err = Range_Int_Iterator_prev(it, &value);
			Debug_assert(DEBUG_THIS, !err, "err = %d\n", err);
			Debug_out(DEBUG_THIS, " - %d\n", value);
		}
		
		Debug_out(DEBUG_THIS, "==== END ROUND %d ======\n", round);
		Debug_out(DEBUG_THIS, "\n");
		Debug_out(DEBUG_THIS, "\n");
	}	
	Range_Int_Iterator_free(&it);

	it = Range_Int_Iterator_new_looping(ri);
	RndGen* rnd = RndGen_new();
	fprintf(stderr, " === RANDOM WALK TEST: START ====\n");
	int forward = 0;
	int steps = 0;
	while(steps <= 50) {
		Range_Int_Iterator_fprint(it, stderr);
		steps++;
		forward = (int)RndGen_nextUL(rnd, 0, 1);
		if (forward) {
			err = Range_Int_Iterator_next(it, &value);
			Debug_assert(DEBUG_THIS, !err, "err = %d\n", err);
			fprintf(stderr, " --> next() --> %d\n", value);
		} else {
			err = Range_Int_Iterator_prev(it, &value);
			Debug_assert(DEBUG_THIS, !err, "err = %d\n", err);
			fprintf(stderr, " --> prev() --> %d\n", value);
		}
	}
	fprintf(stderr, " === RANDOM WALK TEST: END ===");
	
	
	Range_Int_Iterator_free(&it);
	Range_Int_free(&ri);
	return 0;
}
